@@ -22,7 +22,7 @@ from __future__ import annotations
 
 import math
 from abc import ABC, abstractmethod
-from typing import Optional, Union
+from typing import Any, Optional, Union
 
 import torch
 from botorch.acquisition.acquisition import AcquisitionFunction
@@ -116,6 +116,7 @@ class qExpectedImprovement(MCAcquisitionFunction):
         sampler: Optional[MCSampler] = None,
         objective: Optional[MCAcquisitionObjective] = None,
         X_pending: Optional[Tensor] = None,
+        **kwargs: Any,
     ) -> None:
         r"""q-Expected Improvement.
 
@@ -157,7 +158,7 @@ class qExpectedImprovement(MCAcquisitionFunction):
         posterior = self.model.posterior(X)
         samples = self.sampler(posterior)
         obj = self.objective(samples)
-        obj = (obj - self.best_f).clamp_min(0)
+        obj = (obj - self.best_f.unsqueeze(-1)).clamp_min(0)
         q_ei = obj.max(dim=-1)[0].mean(dim=0)
         return q_ei
 
@@ -188,6 +189,7 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
         objective: Optional[MCAcquisitionObjective] = None,
         X_pending: Optional[Tensor] = None,
         prune_baseline: bool = False,
+        **kwargs: Any,
     ) -> None:
         r"""q-Noisy Expected Improvement.
 
@@ -321,7 +323,8 @@ class qProbabilityOfImprovement(MCAcquisitionFunction):
         samples = self.sampler(posterior)
         obj = self.objective(samples)
         max_obj = obj.max(dim=-1)[0]
-        val = torch.sigmoid((max_obj - self.best_f) / self.tau).mean(dim=0)
+        impr = max_obj - self.best_f.unsqueeze(-1)
+        val = torch.sigmoid(impr / self.tau).mean(dim=0)
         return val
 
 
