@@ -11,7 +11,7 @@ from typing import List, Optional
 
 import torch
 from botorch.acquisition.objective import AcquisitionObjective
-from botorch.exceptions.errors import BotorchTensorDimensionError
+from botorch.exceptions.errors import BotorchError, BotorchTensorDimensionError
 from botorch.models.transforms.outcome import Standardize
 from botorch.posteriors import GPyTorchPosterior
 from botorch.utils.transforms import normalize_indices
@@ -19,10 +19,10 @@ from torch import Tensor
 
 
 class MCMultiOutputObjective(AcquisitionObjective):
-    r"""Abstract base class for multi-output objectives."""
+    r"""Abstract base class for MC multi-output objectives."""
 
     @abstractmethod
-    def forward(self, samples: Tensor) -> Tensor:
+    def forward(self, samples: Tensor, **kwargs) -> Tensor:
         r"""Evaluate the multi-output objective on the samples.
 
         Args:
@@ -31,7 +31,8 @@ class MCMultiOutputObjective(AcquisitionObjective):
 
         Returns:
             A `sample_shape x batch_shape x q x m'`-dim Tensor of objective values with
-            `m'` the output dimension. assuming maximization in each output dimension).
+            `m'` the output dimension. This assumes maximization in each output
+            dimension).
 
         This method is usually not called directly, but via the objectives
 
@@ -44,19 +45,87 @@ class MCMultiOutputObjective(AcquisitionObjective):
 
 
 class IdentityMCMultiOutputObjective(MCMultiOutputObjective):
-    r"""Trivial objective extracting the last dimension.
+    r"""Trivial objective that returns the unaltered samples.
 
     Example:
-        >>> identity_objective = IdentityMCObjective()
+        >>> identity_objective = IdentityMCMultiOutputObjective()
         >>> samples = sampler(posterior)
         >>> objective = identity_objective(samples)
     """
 
+    def __init__(
+        self, outcomes: Optional[List[int]] = None, num_outcomes: Optional[int] = None
+    ) -> None:
+        r"""Initialize Objective.
+
+        Args:
+            weights: `m'`-dim tensor of outcome weights.
+            outcomes: A list of the `m'` indices that the weights should be
+                applied to.
+            num_outcomes: The total number of outcomes `m`
+        """
+        super().__init__()
+        if outcomes is not None:
+            if len(outcomes) < 2:
+                raise BotorchTensorDimensionError(
+                    "Must specify at least two outcomes for MOO."
+                )
+            if any(i < 0 for i in outcomes):
+                if num_outcomes is None:
+                    raise BotorchError(
+                        "num_outcomes is required if any outcomes are less than 0."
+                    )
+                outcomes = normalize_indices(outcomes, num_outcomes)
+            self.register_buffer("outcomes", torch.tensor(outcomes, dtype=torch.long))
+
     def forward(self, samples: Tensor) -> Tensor:
+        if hasattr(self, "outcomes"):
+            return samples.index_select(-1, self.outcomes.to(device=samples.device))
         return samples
 
 
-class UnstandardizeMCMultiOutputObjective(MCMultiOutputObjective):
+class WeightedMCMultiOutputObjective(IdentityMCMultiOutputObjective):
+    r"""Objective that reweights samples by given weights vector.
+
+    Example:
+        >>> weights = torch.tensor([1.0, -1.0])
+        >>> weighted_objective = WeightedMCMultiOutputObjective(weights)
+        >>> samples = sampler(posterior)
+        >>> objective = weighted_objective(samples)
+    """
+
+    def __init__(
+        self,
+        weights: Tensor,
+        outcomes: Optional[List[int]] = None,
+        num_outcomes: Optional[int] = None,
+    ) -> None:
+        r"""Initialize Objective.
+
+        Args:
+            weights: `m'`-dim tensor of outcome weights.
+            outcomes: A list of the `m'` indices that the weights should be
+                applied to.
+            num_outcomes: the total number of outcomes `m`
+        """
+        super().__init__(outcomes=outcomes, num_outcomes=num_outcomes)
+        if weights.ndim != 1:
+            raise BotorchTensorDimensionError(
+                f"weights must be an 1-D tensor, but got {weights.shape}."
+            )
+        elif outcomes is not None and weights.shape[0] != len(outcomes):
+            raise BotorchTensorDimensionError(
+                "weights must contain the name number of elements as outcomes, "
+                f"but got {weights.numel()} weights and {len(outcomes)} outcomes."
+            )
+        self.register_buffer("weights", weights)
+
+    def forward(self, samples: Tensor) -> Tensor:
+        samples = super().forward(samples=samples)
+        return samples * self.weights
+
+
+class UnstandardizeMCMultiOutputObjective(IdentityMCMultiOutputObjective):
     r"""Objective that unstandardizes the samples.
 
     TODO: remove this when MultiTask models support outcome transforms.
@@ -84,22 +153,13 @@ class UnstandardizeMCMultiOutputObjective(MCMultiOutputObjective):
                 "Y_mean and Y_std must both be 1-dimensional, but got "
                 f"{Y_mean.ndim} and {Y_std.ndim}"
             )
-        super().__init__()
-        if outcomes is not None:
-            if len(outcomes) < 2:
-                raise BotorchTensorDimensionError(
-                    "Must specify at least two outcomes for MOO."
-                )
-            elif len(outcomes) > Y_mean.shape[-1]:
-                raise BotorchTensorDimensionError(
-                    f"Cannot specify more ({len(outcomes)}) outcomes that present in "
-                    f"the normalization inputs ({Y_mean.shape[-1]})."
-                )
-            nlzd_idcs = normalize_indices(outcomes, Y_mean.shape[-1])
-            self.register_buffer(
-                "outcomes",
-                torch.tensor(nlzd_idcs, dtype=torch.long).to(device=Y_mean.device),
+        elif outcomes is not None and len(outcomes) > Y_mean.shape[-1]:
+            raise BotorchTensorDimensionError(
+                f"Cannot specify more ({len(outcomes)}) outcomes than are present in "
+                f"the normalization inputs ({Y_mean.shape[-1]})."
             )
+        super().__init__(outcomes=outcomes, num_outcomes=Y_mean.shape[-1])
+        if outcomes is not None:
             Y_mean = Y_mean.index_select(-1, self.outcomes)
             Y_std = Y_std.index_select(-1, self.outcomes)
 
@@ -107,8 +167,7 @@ class UnstandardizeMCMultiOutputObjective(MCMultiOutputObjective):
         self.register_buffer("Y_std", Y_std)
 
     def forward(self, samples: Tensor) -> Tensor:
-        if hasattr(self, "outcomes"):
-            samples = samples.index_select(-1, self.outcomes)
+        samples = super().forward(samples=samples)
         return samples * self.Y_std + self.Y_mean
 
 
@@ -143,17 +202,12 @@ class UnstandardizeAnalyticMultiOutputObjective(AnalyticMultiOutputObjective):
         >>> unstd_posterior = unstd_objective(posterior)
     """
 
-    def __init__(
-        self, Y_mean: Tensor, Y_std: Tensor, outcomes: Optional[List[int]] = None
-    ) -> None:
+    def __init__(self, Y_mean: Tensor, Y_std: Tensor) -> None:
         r"""Initialize objective.
 
         Args:
             Y_mean: `m`-dim tensor of outcome means
             Y_std: `m`-dim tensor of outcome standard deviations
-            outcomes: A list of `m' <= m` indices that specifies which of the `m` model
-                outputs should be considered as the outcomes for MOO. If omitted, use
-                all model outcomes. Typically used for constrained optimization.
 
         """
         if Y_mean.ndim > 1 or Y_std.ndim > 1:
@@ -161,20 +215,8 @@ class UnstandardizeAnalyticMultiOutputObjective(AnalyticMultiOutputObjective):
                 "Y_mean and Y_std must both be 1-dimensional, but got "
                 f"{Y_mean.ndim} and {Y_std.ndim}"
             )
-        if outcomes is not None:
-            if len(outcomes) < 2:
-                raise BotorchTensorDimensionError(
-                    "Must specify at least two outcomes for MOO."
-                )
-            elif len(outcomes) > Y_mean.shape[-1]:
-                raise BotorchTensorDimensionError(
-                    f"Cannot specify more ({len(outcomes)}) outcomes that present in "
-                    f"the normalization inputs ({Y_mean.shape[-1]})."
-                )
         super().__init__()
-        self.outcome_transform = Standardize(m=Y_mean.shape[0], outputs=outcomes).to(
-            Y_mean
-        )
+        self.outcome_transform = Standardize(m=Y_mean.shape[0]).to(Y_mean)
         Y_std_unsqueezed = Y_std.unsqueeze(0)
         self.outcome_transform.means = Y_mean.unsqueeze(0)
         self.outcome_transform.stdvs = Y_std_unsqueezed
