@@ -10,11 +10,11 @@ Multi-Task GP models.
 
 from __future__ import annotations
 
-from typing import List, Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple
 
 import torch
 from botorch.models.gpytorch import MultiTaskGPyTorchModel
-from botorch.models.utils import validate_input_scaling
+from botorch.utils.containers import TrainingData
 from gpytorch.distributions.multivariate_normal import MultivariateNormal
 from gpytorch.kernels.index_kernel import IndexKernel
 from gpytorch.kernels.matern_kernel import MaternKernel
@@ -25,6 +25,7 @@ from gpytorch.likelihoods.gaussian_likelihood import (
 )
 from gpytorch.means.constant_mean import ConstantMean
 from gpytorch.models.exact_gp import ExactGP
+from gpytorch.priors.prior import Prior
 from gpytorch.priors.torch_priors import GammaPrior
 from torch import Tensor
 
@@ -47,6 +48,7 @@ class MultiTaskGP(ExactGP, MultiTaskGPyTorchModel):
         train_X: Tensor,
         train_Y: Tensor,
         task_feature: int,
+        prior: Optional[Prior] = None,
         output_tasks: Optional[List[int]] = None,
         rank: Optional[int] = None,
     ) -> None:
@@ -56,10 +58,8 @@ class MultiTaskGP(ExactGP, MultiTaskGPyTorchModel):
             train_X: A `n x (d + 1)` or `b x n x (d + 1)` (batch mode) tensor
                 of training data. One of the columns should contain the task
                 features (see `task_feature` argument).
-            train_Y: A `n` or `b x n` (batch mode) tensor of training
-                observations.
-            task_feature: The index of the task feature
-                (`-d <= task_feature <= d`).
+            train_Y: A `n` or `b x n` (batch mode) tensor of training observations.
+            task_feature: The index of the task feature (`-d <= task_feature <= d`).
             output_tasks: A list of task indices for which to compute model
                 outputs for. If omitted, return outputs for all task indices.
             rank: The rank to be used for the index kernel. If omitted, use a
@@ -74,21 +74,15 @@ class MultiTaskGP(ExactGP, MultiTaskGPyTorchModel):
             >>> train_Y = torch.cat(f1(X1), f2(X2)).unsqueeze(-1)
             >>> model = MultiTaskGP(train_X, train_Y, task_feature=-1)
         """
-        self._validate_tensor_args(X=train_X, Y=train_Y)
-        validate_input_scaling(train_X=train_X, train_Y=train_Y)
-        if train_X.ndim != 2:
-            # Currently, batch mode MTGPs are blocked upstream in GPyTorch
-            raise ValueError(f"Unsupported shape {train_X.shape} for train_X.")
+        all_tasks, task_feature, d = self.get_all_tasks(
+            train_X, task_feature, output_tasks
+        )
         # squeeze output dim
         train_Y = train_Y.squeeze(-1)
-        d = train_X.shape[-1] - 1
-        if not (-d <= task_feature <= d):
-            raise ValueError(f"Must have that -{d} <= task_feature <= {d}")
-        all_tasks = train_X[:, task_feature].unique().to(dtype=torch.long).tolist()
         if output_tasks is None:
             output_tasks = all_tasks
         else:
-            if any(t not in all_tasks for t in output_tasks):
+            if set(output_tasks) - set(all_tasks):
                 raise RuntimeError("All output tasks must be present in input data.")
         self._output_tasks = output_tasks
         self._num_outputs = len(output_tasks)
@@ -113,8 +107,10 @@ class MultiTaskGP(ExactGP, MultiTaskGPyTorchModel):
         )
         num_tasks = len(all_tasks)
         self._rank = rank if rank is not None else num_tasks
-        # TODO: Add LKJ prior for the index kernel
-        self.task_covar_module = IndexKernel(num_tasks=num_tasks, rank=self._rank)
+
+        self.task_covar_module = IndexKernel(
+            num_tasks=num_tasks, rank=self._rank, prior=prior
+        )
         self.to(train_X)
 
     def _split_inputs(self, x: Tensor) -> Tuple[Tensor, Tensor]:
@@ -154,6 +150,46 @@ class MultiTaskGP(ExactGP, MultiTaskGPyTorchModel):
         covar = covar_x.mul(covar_i)
         return MultivariateNormal(mean_x, covar)
 
+    @classmethod
+    def get_all_tasks(
+        cls,
+        train_X: Tensor,
+        task_feature: int,
+        output_tasks: Optional[List[int]] = None,
+    ) -> Tuple[List[int], int, int]:
+        if train_X.ndim != 2:
+            # Currently, batch mode MTGPs are blocked upstream in GPyTorch
+            raise ValueError(f"Unsupported shape {train_X.shape} for train_X.")
+        d = train_X.shape[-1] - 1
+        if not (-d <= task_feature <= d):
+            raise ValueError(f"Must have that -{d} <= task_feature <= {d}")
+        task_feature = task_feature % (d + 1)
+        all_tasks = train_X[:, task_feature].unique().to(dtype=torch.long).tolist()
+        return all_tasks, task_feature, d
+
+    @classmethod
+    def construct_inputs(cls, training_data: TrainingData, **kwargs) -> Dict[str, Any]:
+        r"""Construct kwargs for the `Model` from `TrainingData`.
+
+        Args:
+            training_data: `TrainingData` container with data for single outcome
+                or for multiple outcomes for batched multi-output case.
+            **kwargs: Additional options for the model that pertain to the
+                training data:
+                - `task_features` – indices of the input columns containing the task
+                features (expected list of length 1).
+        """
+
+        task_features = kwargs.get("task_features")
+        if task_features is None:
+            raise ValueError(f"task features required for {cls.__name__}.")
+
+        return {
+            "train_X": training_data.X,
+            "train_Y": training_data.Y,
+            "task_feature": task_features[0],
+        }
+
 
 class FixedNoiseMultiTaskGP(MultiTaskGP):
     r"""Multi-Task GP model using an ICM kernel, with known observation noise.
@@ -172,6 +208,7 @@ class FixedNoiseMultiTaskGP(MultiTaskGP):
         train_Y: Tensor,
         train_Yvar: Tensor,
         task_feature: int,
+        prior: Optional[Prior] = None,
         output_tasks: Optional[List[int]] = None,
         rank: Optional[int] = None,
     ) -> None:
@@ -185,8 +222,7 @@ class FixedNoiseMultiTaskGP(MultiTaskGP):
                 observations.
             train_Yvar: A `n` or `b x n` (batch mode) tensor of observation
                 noise standard errors.
-            task_feature: The index of the task feature
-                (`-d <= task_feature <= d`).
+            task_feature: The index of the task feature (`-d <= task_feature <= d`).
             output_tasks: A list of task indices for which to compute model
                 outputs for. If omitted, return outputs for all task indices.
             rank: The rank to be used for the index kernel. If omitted, use a
@@ -210,6 +246,32 @@ class FixedNoiseMultiTaskGP(MultiTaskGP):
             task_feature=task_feature,
             output_tasks=output_tasks,
             rank=rank,
+            prior=prior,
         )
         self.likelihood = FixedNoiseGaussianLikelihood(noise=train_Yvar.squeeze(-1))
         self.to(train_X)
+
+    @classmethod
+    def construct_inputs(cls, training_data: TrainingData, **kwargs) -> Dict[str, Any]:
+        r"""Construct kwargs for the `Model` from `TrainingData`.
+
+        Args:
+            training_data: `TrainingData` container with data for single outcome
+                or for multiple outcomes for batched multi-output case.
+            **kwargs: Additional options for the model that pertain to the
+                training data:
+                - `task_features` – indices of task features in X.
+        """
+
+        task_features = kwargs.get("task_features")
+        if task_features is None:
+            raise ValueError(f"task features required for {cls.__name__}.")
+        if training_data.Yvar is None:
+            raise ValueError(f"Yvar required for {cls.__name__}.")
+
+        return {
+            "train_X": training_data.X,
+            "train_Y": training_data.Y,
+            "train_Yvar": training_data.Yvar,
+            "task_feature": task_features[0],
+        }
