@@ -4,9 +4,10 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from .constraints import get_outcome_constraint_transforms
-from .objective import apply_constraints, get_objective_weights_transform
-from .sampling import (
+from botorch.utils.constraints import get_outcome_constraint_transforms
+from botorch.utils.feasible_volume import estimate_feasible_volume
+from botorch.utils.objective import apply_constraints, get_objective_weights_transform
+from botorch.utils.sampling import (
     batched_multinomial,
     construct_base_samples,
     construct_base_samples_from_posterior,
@@ -14,7 +15,11 @@ from .sampling import (
     draw_sobol_samples,
     manual_seed,
 )
-from .transforms import squeeze_last_dim, standardize, t_batch_mode_transform
+from botorch.utils.transforms import (
+    squeeze_last_dim,
+    standardize,
+    t_batch_mode_transform,
+)
 
 
 __all__ = [
@@ -30,4 +35,5 @@ __all__ = [
     "squeeze_last_dim",
     "standardize",
     "t_batch_mode_transform",
+    "estimate_feasible_volume",
 ]
