@@ -12,6 +12,7 @@ from botorch.exceptions.warnings import OptimizationWarning
 from botorch.fit import fit_gpytorch_model
 from botorch.models.multitask import FixedNoiseMultiTaskGP, MultiTaskGP
 from botorch.posteriors import GPyTorchPosterior
+from botorch.utils.containers import TrainingData
 from botorch.utils.testing import BotorchTestCase
 from gpytorch.distributions import MultitaskMultivariateNormal, MultivariateNormal
 from gpytorch.kernels import IndexKernel, MaternKernel, ScaleKernel
@@ -19,6 +20,7 @@ from gpytorch.likelihoods import FixedNoiseGaussianLikelihood, GaussianLikelihoo
 from gpytorch.means import ConstantMean
 from gpytorch.mlls.exact_marginal_log_likelihood import ExactMarginalLogLikelihood
 from gpytorch.priors import GammaPrior
+from gpytorch.priors.lkj_prior import LKJCovariancePrior
 
 
 def _get_random_mt_data(**tkwargs):
@@ -36,9 +38,13 @@ def _get_random_mt_data(**tkwargs):
 
 
 def _get_model(**tkwargs):
+    return _get_model_and_training_data(**tkwargs)[0]
+
+
+def _get_model_and_training_data(**tkwargs):
     train_X, train_Y = _get_random_mt_data(**tkwargs)
     model = MultiTaskGP(train_X, train_Y, task_feature=1)
-    return model.to(**tkwargs)
+    return model.to(**tkwargs), train_X, train_Y
 
 
 def _get_model_single_output(**tkwargs):
@@ -48,10 +54,14 @@ def _get_model_single_output(**tkwargs):
 
 
 def _get_fixed_noise_model(**tkwargs):
+    return _get_fixed_noise_model_and_training_data(**tkwargs)[0]
+
+
+def _get_fixed_noise_model_and_training_data(**tkwargs):
     train_X, train_Y = _get_random_mt_data(**tkwargs)
     train_Yvar = torch.full_like(train_Y, 0.05)
     model = FixedNoiseMultiTaskGP(train_X, train_Y, train_Yvar, task_feature=1)
-    return model.to(**tkwargs)
+    return model.to(**tkwargs), train_X, train_Y, train_Yvar
 
 
 def _get_fixed_noise_model_single_output(**tkwargs):
@@ -63,6 +73,31 @@ def _get_fixed_noise_model_single_output(**tkwargs):
     return model.to(**tkwargs)
 
 
+def _get_fixed_prior_model(**tkwargs):
+    train_X, train_Y = _get_random_mt_data(**tkwargs)
+    sd_prior = GammaPrior(2.0, 0.15)
+    sd_prior._event_shape = torch.Size([2])
+    model = MultiTaskGP(
+        train_X, train_Y, task_feature=1, prior=LKJCovariancePrior(2, 0.6, sd_prior)
+    )
+    return model.to(**tkwargs)
+
+
+def _get_fixed_noise_and_prior_model(**tkwargs):
+    train_X, train_Y = _get_random_mt_data(**tkwargs)
+    train_Yvar = torch.full_like(train_Y, 0.05)
+    sd_prior = GammaPrior(2.0, 0.15)
+    sd_prior._event_shape = torch.Size([2])
+    model = FixedNoiseMultiTaskGP(
+        train_X,
+        train_Y,
+        train_Yvar,
+        task_feature=1,
+        prior=LKJCovariancePrior(2, 0.6, sd_prior),
+    )
+    return model.to(**tkwargs)
+
+
 class TestMultiTaskGP(BotorchTestCase):
     def test_MultiTaskGP(self):
         for dtype in (torch.float, torch.double):
@@ -173,6 +208,16 @@ class TestMultiTaskGP(BotorchTestCase):
             self.assertIsInstance(posterior_f, GPyTorchPosterior)
             self.assertIsInstance(posterior_f.mvn, MultivariateNormal)
 
+    def test_MultiTaskGP_fixed_prior(self):
+        for dtype in (torch.float, torch.double):
+            tkwargs = {"device": self.device, "dtype": dtype}
+            model = _get_fixed_prior_model(**tkwargs)
+            self.assertIsInstance(model, MultiTaskGP)
+            self.assertIsInstance(model.task_covar_module, IndexKernel)
+            self.assertIsInstance(
+                model.task_covar_module.IndexKernelPrior, LKJCovariancePrior
+            )
+
 
 class TestFixedNoiseMultiTaskGP(BotorchTestCase):
     def test_FixedNoiseMultiTaskGP(self):
@@ -281,3 +326,46 @@ class TestFixedNoiseMultiTaskGP(BotorchTestCase):
             posterior_f = model.posterior(test_x)
             self.assertIsInstance(posterior_f, GPyTorchPosterior)
             self.assertIsInstance(posterior_f.mvn, MultivariateNormal)
+
+    def test_FixedNoiseMultiTaskGP_fixed_prior(self):
+        for dtype in (torch.float, torch.double):
+            tkwargs = {"device": self.device, "dtype": dtype}
+            model = _get_fixed_noise_and_prior_model(**tkwargs)
+            self.assertIsInstance(model, FixedNoiseMultiTaskGP)
+            self.assertIsInstance(model.task_covar_module, IndexKernel)
+            self.assertIsInstance(
+                model.task_covar_module.IndexKernelPrior, LKJCovariancePrior
+            )
+
+    def test_MultiTaskGP_construct_inputs(self):
+        for dtype in (torch.float, torch.double):
+            tkwargs = {"device": self.device, "dtype": dtype}
+            model, train_X, train_Y = _get_model_and_training_data(**tkwargs)
+            training_data = TrainingData(X=train_X, Y=train_Y)
+            # if task_features is missing, then raise error
+            with self.assertRaises(ValueError):
+                model.construct_inputs(training_data)
+            data_dict = model.construct_inputs(training_data, task_features=[0])
+            self.assertTrue(torch.equal(data_dict["train_X"], train_X))
+            self.assertTrue(torch.equal(data_dict["train_Y"], train_Y))
+
+    def test_FixedNoiseMultiTaskGP_construct_inputs(self):
+        for dtype in (torch.float, torch.double):
+            tkwargs = {"device": self.device, "dtype": dtype}
+            (
+                model,
+                train_X,
+                train_Y,
+                train_Yvar,
+            ) = _get_fixed_noise_model_and_training_data(**tkwargs)
+            td_no_Yvar = TrainingData(X=train_X, Y=train_Y)
+            # if task_features is missing, then raise error
+            with self.assertRaisesRegex(ValueError, "task features required"):
+                model.construct_inputs(td_no_Yvar)
+            # if task_features is missing, then raise error
+            with self.assertRaisesRegex(ValueError, "Yvar required"):
+                model.construct_inputs(td_no_Yvar, task_features=[0])
+            training_data = TrainingData(X=train_X, Y=train_Y, Yvar=train_Yvar)
+            data_dict = model.construct_inputs(training_data, task_features=[0])
+            self.assertTrue(torch.equal(data_dict["train_X"], train_X))
+            self.assertTrue(torch.equal(data_dict["train_Y"], train_Y))
