@@ -4,6 +4,8 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from typing import Any
+
 import torch
 from botorch.utils.testing import BotorchTestCase
 from botorch.utils.transforms import (
@@ -70,7 +72,7 @@ class TestNormalizeAndUnnormalize(BotorchTestCase):
             self.assertTrue(torch.equal(X2, unnormalize(X2_normalized, bounds=bounds2)))
 
 
-class BMIMTestClass:
+class BMIMTestClass(BotorchTestCase):
     @t_batch_mode_transform()
     def q_method(self, X: Tensor) -> None:
         return X
@@ -79,6 +81,11 @@ class BMIMTestClass:
     def q1_method(self, X: Tensor) -> None:
         return X
 
+    @t_batch_mode_transform()
+    def kw_method(self, X: Tensor, dummy_arg: Any = None):
+        self.assertIsNotNone(dummy_arg)
+        return X
+
     @concatenate_pending_points
     def dummy_method(self, X: Tensor) -> Tensor:
         return X
@@ -123,6 +130,13 @@ class TestBatchModeTransform(BotorchTestCase):
         with self.assertRaises(ValueError):
             c.q_method(X)
 
+        # test with kwargs
+        X = torch.rand(1, 2)
+        with self.assertRaises(AssertionError):
+            c.kw_method(X)
+        Xout = c.kw_method(X, dummy_arg=5)
+        self.assertTrue(torch.equal(Xout, X.unsqueeze(0)))
+
 
 class TestConcatenatePendingPoints(BotorchTestCase):
     def test_concatenate_pending_points(self):
